const TokenizeCodeBlock = 'Tokenizer.tokenizeCodeBlock';
const LoadTokenizer = 'Tokenizer.load';
const HandleMessagePort = 'HandleMessagePort.handleMessagePort';
const GetTokensViewport = 'GetTokensViewport.getTokensViewport';
const SetLines = 'TextDocument.setLines';

const deepCopy = value => {
  return structuredClone(value);
};

const getInitialLineState = initialLineState => {
  return deepCopy(initialLineState);
};

let AssertionError$1 = class AssertionError extends Error {
  constructor(message) {
    super(message);
    this.name = 'AssertionError';
  }
};
const getType$1 = value => {
  switch (typeof value) {
    case 'number':
      return 'number';
    case 'function':
      return 'function';
    case 'string':
      return 'string';
    case 'object':
      if (value === null) {
        return 'null';
      }
      if (Array.isArray(value)) {
        return 'array';
      }
      return 'object';
    case 'boolean':
      return 'boolean';
    default:
      return 'unknown';
  }
};
const object = value => {
  const type = getType$1(value);
  if (type !== 'object') {
    throw new AssertionError$1('expected value to be of type object');
  }
};

const state = {
  warned: []
};
const flattenTokensArray = tokens => {
  const flattened = [];
  for (const token of tokens) {
    object(token);
    flattened.push(token.type, token.length);
  }
  return flattened;
};
const warnDeprecatedArrayReturn = (languageId, fn) => {
  if (state.warned.includes(fn)) {
    return;
  }
  state.warned.push(fn);
  console.warn(`tokenizers without hasArrayReturn=false are deprecated (language ${languageId})`);
};
/**
 *
 * @param {(line:string, lineState)=>any} tokenizeLine
 * @param {string} line
 * @param {any} lineStateAtStart
 * @param {boolean} hasArrayReturn
 * @returns
 */
const safeTokenizeLine = (languageId, tokenizeLine, line, lineStateAtStart, hasArrayReturn) => {
  try {
    const lineState = tokenizeLine(line, lineStateAtStart);
    if (!lineState || !lineState.tokens || !lineState.state) {
      throw new Error('invalid tokenization result');
    }
    if (!hasArrayReturn) {
      warnDeprecatedArrayReturn(languageId, tokenizeLine);
      // workaround for old tokenizers
      lineState.tokens = flattenTokensArray(lineState.tokens);
    }
    return lineState;
  } catch (error) {
    console.error(error);
    return {
      tokens: [/* type */0, /* length */line.length],
      lineState: lineStateAtStart
    };
  }
};

/**
 * @enum number
 */
const State = {
  TopLevelContent: 1
};

/**
 * @enum number
 */
const TokenType = {
  Text: 1
};
const TokenMap = {
  [TokenType.Text]: 'Text'
};
const initialLineState = {
  state: State.TopLevelContent
};
const hasArrayReturn = true;
const tokenizeLine = (line, lineState) => {
  return {
    tokens: [TokenType.Text, line.length],
    state: lineState.state
  };
};

const TokenizePlainText = {
  __proto__: null,
  State,
  TokenMap,
  TokenType,
  hasArrayReturn,
  initialLineState,
  tokenizeLine
};

const tokenizers = Object.create(null);
const set = (id, value) => {
  tokenizers[id] = value;
};
const get$1 = id => {
  return tokenizers[id] || TokenizePlainText;
};

// TODO loadTokenizer should be invoked from renderer worker
const loadTokenizer = async (languageId, tokenizePath) => {
  if (!tokenizePath) {
    return;
  }
  try {
    // TODO check that tokenizer is valid
    // 1. tokenizeLine should be of type function
    // 2. getTokenClass should be of type function
    const tokenizer = await import(tokenizePath);
    if (typeof tokenizer.tokenizeLine !== 'function') {
      console.warn(`tokenizer.tokenizeLine should be a function in "${tokenizePath}"`);
      return;
    }
    if (!tokenizer.TokenMap || typeof tokenizer.TokenMap !== 'object' || Array.isArray(tokenizer.TokenMap)) {
      console.warn(`tokenizer.TokenMap should be an object in "${tokenizePath}"`);
      return;
    }
    set(languageId, tokenizer);
    return tokenizer.TokenMap;
  } catch (error) {
    // TODO better error handling
    console.error(error);
    return;
  }
};
const getTokenizer = languageId => {
  if (get$1(languageId)) {
    return get$1(languageId);
  }
  return TokenizePlainText;
};

const getTokensViewportEmbedded = (langageId, lines, lineCache, linesWithEmbed) => {
  const tokenizersToLoad = [];
  const embeddedResults = [];
  let topContext;
  for (const index of linesWithEmbed) {
    const result = lineCache[index + 1];
    const line = lines[index];
    if (result.embeddedLanguage) {
      const {
        embeddedLanguage,
        embeddedLanguageStart,
        embeddedLanguageEnd
      } = result;
      const embeddedTokenizer = getTokenizer(embeddedLanguage);
      if (embeddedLanguageStart !== line.length && embeddedTokenizer && embeddedTokenizer !== TokenizePlainText) {
        const isFull = embeddedLanguageStart === 0 && embeddedLanguageEnd === line.length;
        const partialLine = line.slice(embeddedLanguageStart, embeddedLanguageEnd);
        const embedResult = safeTokenizeLine(langageId, embeddedTokenizer.tokenizeLine, partialLine, topContext || getInitialLineState(embeddedTokenizer.initialLineState), embeddedTokenizer.hasArrayReturn);
        topContext = embedResult;
        result.embeddedResultIndex = embeddedResults.length;
        embeddedResults.push({
          result: embedResult,
          TokenMap: embeddedTokenizer.TokenMap,
          isFull
        });
      } else if (line.length === 0) {
        const embedResult = {
          tokens: []
        };
        result.embeddedResultIndex = embeddedResults.length;
        embeddedResults.push({
          result: embedResult,
          isFull: true,
          TokenMap: []
        });
      } else {
        tokenizersToLoad.push(embeddedLanguage);
        embeddedResults.push({
          result: {},
          isFull: false,
          TokenMap: []
        });
        topContext = undefined;
      }
    } else {
      topContext = undefined;
    }
  }
  return {
    tokenizersToLoad,
    embeddedResults
  };
};
const getTokenizeEndIndex = (invalidStartIndex, endLineIndex, tokenizeStartIndex) => {
  return invalidStartIndex < endLineIndex ? endLineIndex : tokenizeStartIndex;
};

// TODO only send changed lines to renderer process instead of all lines in viewport
const getTokensViewport = (editor, startLineIndex, endLineIndex) => {
  const {
    invalidStartIndex,
    lines,
    languageId
  } = editor;
  const lineCache = [];
  const tokenizer = get$1(languageId);
  const {
    hasArrayReturn,
    tokenizeLine,
    initialLineState
  } = tokenizer;
  const tokenizeStartIndex = invalidStartIndex;
  const tokenizeEndIndex = getTokenizeEndIndex(invalidStartIndex, endLineIndex, tokenizeStartIndex);
  const tokenizersToLoad = [];
  const embeddedResults = [];
  const linesWithEmbed = [];
  for (let i = tokenizeStartIndex; i < tokenizeEndIndex; i++) {
    const lineState = i === 0 ? getInitialLineState(initialLineState) : lineCache[i];
    const line = lines[i];
    const result = safeTokenizeLine(languageId, tokenizeLine, line, lineState, hasArrayReturn);
    // TODO if lineCacheEnd matches the one before, skip tokenizing lines after
    lineCache[i + 1] = result;
    if (result.embeddedLanguage) {
      result.embeddedResultIndex = linesWithEmbed.length;
      linesWithEmbed.push(i);
    }
  }
  const visibleLines = lineCache.slice(startLineIndex + 1, endLineIndex + 1);
  if (linesWithEmbed.length > 0) {
    const {
      tokenizersToLoad,
      embeddedResults
    } = getTokensViewportEmbedded(languageId, lines, lineCache, linesWithEmbed);
    // TODO support lineCache with embedded content
    editor.invalidStartIndex = 0;
    return {
      tokens: visibleLines,
      tokenizersToLoad,
      embeddedResults
    };
  }
  editor.invalidStartIndex = Math.max(invalidStartIndex, tokenizeEndIndex);
  return {
    tokens: visibleLines,
    tokenizersToLoad,
    embeddedResults
  };
};

const Two = '2.0';
class AssertionError extends Error {
  constructor(message) {
    super(message);
    this.name = 'AssertionError';
  }
}
const getType = value => {
  switch (typeof value) {
    case 'number':
      return 'number';
    case 'function':
      return 'function';
    case 'string':
      return 'string';
    case 'object':
      if (value === null) {
        return 'null';
      }
      if (Array.isArray(value)) {
        return 'array';
      }
      return 'object';
    case 'boolean':
      return 'boolean';
    default:
      return 'unknown';
  }
};
const number = value => {
  const type = getType(value);
  if (type !== 'number') {
    throw new AssertionError('expected value to be of type number');
  }
};
const state$1 = {
  callbacks: Object.create(null)
};
const get = id => {
  return state$1.callbacks[id];
};
const remove = id => {
  delete state$1.callbacks[id];
};
const warn = (...args) => {
  console.warn(...args);
};
const resolve = (id, args) => {
  number(id);
  const fn = get(id);
  if (!fn) {
    console.log(args);
    warn(`callback ${id} may already be disposed`);
    return;
  }
  fn(args);
  remove(id);
};
class JsonRpcError extends Error {
  constructor(message) {
    super(message);
    this.name = 'JsonRpcError';
  }
}
const MethodNotFound = -32601;
const Custom = -32001;
const create$1 = (message, error) => {
  return {
    jsonrpc: Two,
    id: message.id,
    error
  };
};
const E_COMMAND_NOT_FOUND = 'E_COMMAND_NOT_FOUND';
const getErrorProperty = (error, prettyError) => {
  if (error && error.code === E_COMMAND_NOT_FOUND) {
    return {
      code: MethodNotFound,
      message: error.message,
      data: error.stack
    };
  }
  return {
    code: Custom,
    message: prettyError.message,
    data: {
      stack: prettyError.stack,
      codeFrame: prettyError.codeFrame,
      type: prettyError.type,
      code: prettyError.code
    }
  };
};
const getErrorResponse = (message, error, preparePrettyError, logError) => {
  const prettyError = preparePrettyError(error);
  logError(error, prettyError);
  const errorProperty = getErrorProperty(error, prettyError);
  return create$1(message, errorProperty);
};
const create = (message, result) => {
  return {
    jsonrpc: Two,
    id: message.id,
    result: result ?? null
  };
};
const getSuccessResponse = (message, result) => {
  const resultProperty = result ?? null;
  return create(message, resultProperty);
};
const getResponse = async (message, ipc, execute, preparePrettyError, logError, requiresSocket) => {
  try {
    const result = requiresSocket(message.method) ? await execute(message.method, ipc, ...message.params) : await execute(message.method, ...message.params);
    return getSuccessResponse(message, result);
  } catch (error) {
    return getErrorResponse(message, error, preparePrettyError, logError);
  }
};
const handleJsonRpcMessage = async (ipc, message, execute, resolve, preparePrettyError, logError, requiresSocket) => {
  if ('id' in message) {
    if ('method' in message) {
      const response = await getResponse(message, ipc, execute, preparePrettyError, logError, requiresSocket);
      try {
        ipc.send(response);
      } catch (error) {
        const errorResponse = getErrorResponse(message, error, preparePrettyError, logError);
        ipc.send(errorResponse);
      }
      return;
    }
    resolve(message.id, message);
    return;
  }
  if ('method' in message) {
    await getResponse(message, ipc, execute, preparePrettyError, logError, requiresSocket);
    return;
  }
  throw new JsonRpcError('unexpected message');
};

const commands = Object.create(null);
const registerCommand = (key, fn) => {
  commands[key] = fn;
};
const registerCommands = commandMap => {
  for (const [key, value] of Object.entries(commandMap)) {
    registerCommand(key, value);
  }
};
const getCommand = key => {
  return commands[key];
};

const processName = 'syntax-highlighting-worker';

const execute = (command, ...args) => {
  const fn = getCommand(command);
  if (!fn) {
    throw new Error(`[${processName}] command not found ${command}`);
  }
  return fn(...args);
};

const requiresSocket = () => {
  return false;
};
const preparePrettyError = error => {
  return error;
};
const logError = error => {
  console.error(error);
};
const handleMessage = event => {
  return handleJsonRpcMessage(event.target, event.data, execute, resolve, preparePrettyError, logError, requiresSocket);
};

const handleIpc = ipc => {
  ipc.addEventListener('message', handleMessage);
};

const MessagePort = 1;
const ModuleWorker = 2;
const ReferencePort = 3;
const ModuleWorkerAndMessagePort = 8;
const Auto = () => {
  if (globalThis.acceptPort) {
    return MessagePort;
  }
  if (globalThis.acceptReferencePort) {
    return ReferencePort;
  }
  return ModuleWorkerAndMessagePort;
};

const getData$1 = event => {
  return event.data;
};
const attachEvents = that => {
  const handleMessage = (...args) => {
    const data = that.getData(...args);
    that.dispatchEvent(new MessageEvent('message', {
      data
    }));
  };
  that.onMessage(handleMessage);
  const handleClose = event => {
    that.dispatchEvent(new Event('close'));
  };
  that.onClose(handleClose);
};
class Ipc extends EventTarget {
  constructor(rawIpc) {
    super();
    this._rawIpc = rawIpc;
    attachEvents(this);
  }
}
const readyMessage = 'ready';
const listen$3 = () => {
  // @ts-ignore
  if (typeof WorkerGlobalScope === 'undefined') {
    throw new TypeError('module is not in web worker scope');
  }
  return globalThis;
};
const signal$2 = global => {
  global.postMessage(readyMessage);
};
class IpcChildWithModuleWorker extends Ipc {
  getData(event) {
    return getData$1(event);
  }
  send(message) {
    // @ts-ignore
    this._rawIpc.postMessage(message);
  }
  sendAndTransfer(message, transfer) {
    // @ts-ignore
    this._rawIpc.postMessage(message, transfer);
  }
  dispose() {
    // ignore
  }
  onClose(callback) {
    // ignore
  }
  onMessage(callback) {
    this._rawIpc.addEventListener('message', callback);
  }
}
const wrap$5 = global => {
  return new IpcChildWithModuleWorker(global);
};
const IpcChildWithModuleWorker$1 = {
  __proto__: null,
  listen: listen$3,
  signal: signal$2,
  wrap: wrap$5
};
const E_INCOMPATIBLE_NATIVE_MODULE = 'E_INCOMPATIBLE_NATIVE_MODULE';
const E_MODULES_NOT_SUPPORTED_IN_ELECTRON = 'E_MODULES_NOT_SUPPORTED_IN_ELECTRON';
const ERR_MODULE_NOT_FOUND = 'ERR_MODULE_NOT_FOUND';
const NewLine$1 = '\n';
const joinLines = lines => {
  return lines.join(NewLine$1);
};
const splitLines$1 = lines => {
  return lines.split(NewLine$1);
};
const RE_NATIVE_MODULE_ERROR = /^innerError Error: Cannot find module '.*.node'/;
const RE_NATIVE_MODULE_ERROR_2 = /was compiled against a different Node.js version/;
const RE_MESSAGE_CODE_BLOCK_START = /^Error: The module '.*'$/;
const RE_MESSAGE_CODE_BLOCK_END = /^\s* at/;
const RE_AT = /^\s+at/;
const RE_AT_PROMISE_INDEX = /^\s*at async Promise.all \(index \d+\)$/;
const isUnhelpfulNativeModuleError = stderr => {
  return RE_NATIVE_MODULE_ERROR.test(stderr) && RE_NATIVE_MODULE_ERROR_2.test(stderr);
};
const isMessageCodeBlockStartIndex = line => {
  return RE_MESSAGE_CODE_BLOCK_START.test(line);
};
const isMessageCodeBlockEndIndex = line => {
  return RE_MESSAGE_CODE_BLOCK_END.test(line);
};
const getMessageCodeBlock = stderr => {
  const lines = splitLines$1(stderr);
  const startIndex = lines.findIndex(isMessageCodeBlockStartIndex);
  const endIndex = startIndex + lines.slice(startIndex).findIndex(isMessageCodeBlockEndIndex, startIndex);
  const relevantLines = lines.slice(startIndex, endIndex);
  const relevantMessage = relevantLines.join(' ').slice('Error: '.length);
  return relevantMessage;
};
const getNativeModuleErrorMessage = stderr => {
  const message = getMessageCodeBlock(stderr);
  return {
    message: `Incompatible native node module: ${message}`,
    code: E_INCOMPATIBLE_NATIVE_MODULE
  };
};
const isModulesSyntaxError = stderr => {
  if (!stderr) {
    return false;
  }
  return stderr.includes('SyntaxError: Cannot use import statement outside a module');
};
const getModuleSyntaxError = () => {
  return {
    message: `ES Modules are not supported in electron`,
    code: E_MODULES_NOT_SUPPORTED_IN_ELECTRON
  };
};
const isModuleNotFoundError = stderr => {
  if (!stderr) {
    return false;
  }
  return stderr.includes('ERR_MODULE_NOT_FOUND');
};
const isModuleNotFoundMessage = line => {
  return line.includes('ERR_MODULE_NOT_FOUND');
};
const getModuleNotFoundError = stderr => {
  const lines = splitLines$1(stderr);
  const messageIndex = lines.findIndex(isModuleNotFoundMessage);
  const message = lines[messageIndex];
  return {
    message,
    code: ERR_MODULE_NOT_FOUND
  };
};
const isNormalStackLine = line => {
  return RE_AT.test(line) && !RE_AT_PROMISE_INDEX.test(line);
};
const getDetails = lines => {
  const index = lines.findIndex(isNormalStackLine);
  if (index === -1) {
    return {
      actualMessage: joinLines(lines),
      rest: []
    };
  }
  let lastIndex = index - 1;
  while (++lastIndex < lines.length) {
    if (!isNormalStackLine(lines[lastIndex])) {
      break;
    }
  }
  return {
    actualMessage: lines[index - 1],
    rest: lines.slice(index, lastIndex)
  };
};
const getHelpfulChildProcessError = (stdout, stderr) => {
  if (isUnhelpfulNativeModuleError(stderr)) {
    return getNativeModuleErrorMessage(stderr);
  }
  if (isModulesSyntaxError(stderr)) {
    return getModuleSyntaxError();
  }
  if (isModuleNotFoundError(stderr)) {
    return getModuleNotFoundError(stderr);
  }
  const lines = splitLines$1(stderr);
  const {
    actualMessage,
    rest
  } = getDetails(lines);
  return {
    message: `${actualMessage}`,
    code: '',
    stack: rest
  };
};
const normalizeLine = line => {
  if (line.startsWith('Error: ')) {
    return line.slice(`Error: `.length);
  }
  if (line.startsWith('VError: ')) {
    return line.slice(`VError: `.length);
  }
  return line;
};
const getCombinedMessage = (error, message) => {
  const stringifiedError = normalizeLine(`${error}`);
  if (message) {
    return `${message}: ${stringifiedError}`;
  }
  return stringifiedError;
};
const NewLine$2 = '\n';
const getNewLineIndex = (string, startIndex = undefined) => {
  return string.indexOf(NewLine$2, startIndex);
};
const mergeStacks = (parent, child) => {
  if (!child) {
    return parent;
  }
  const parentNewLineIndex = getNewLineIndex(parent);
  const childNewLineIndex = getNewLineIndex(child);
  if (childNewLineIndex === -1) {
    return parent;
  }
  const parentFirstLine = parent.slice(0, parentNewLineIndex);
  const childRest = child.slice(childNewLineIndex);
  const childFirstLine = normalizeLine(child.slice(0, childNewLineIndex));
  if (parentFirstLine.includes(childFirstLine)) {
    return parentFirstLine + childRest;
  }
  return child;
};
class VError extends Error {
  constructor(error, message) {
    const combinedMessage = getCombinedMessage(error, message);
    super(combinedMessage);
    this.name = 'VError';
    if (error instanceof Error) {
      this.stack = mergeStacks(this.stack, error.stack);
    }
    if (error.codeFrame) {
      // @ts-ignore
      this.codeFrame = error.codeFrame;
    }
    if (error.code) {
      // @ts-ignore
      this.code = error.code;
    }
  }
}
class IpcError extends VError {
  // @ts-ignore
  constructor(betterMessage, stdout = '', stderr = '') {
    if (stdout || stderr) {
      // @ts-ignore
      const {
        message,
        code,
        stack
      } = getHelpfulChildProcessError(stdout, stderr);
      const cause = new Error(message);
      // @ts-ignore
      cause.code = code;
      cause.stack = stack;
      super(cause, betterMessage);
    } else {
      super(betterMessage);
    }
    // @ts-ignore
    this.name = 'IpcError';
    // @ts-ignore
    this.stdout = stdout;
    // @ts-ignore
    this.stderr = stderr;
  }
}
const withResolvers = () => {
  let _resolve;
  const promise = new Promise(resolve => {
    _resolve = resolve;
  });
  return {
    resolve: _resolve,
    promise
  };
};
const waitForFirstMessage = async port => {
  const {
    resolve,
    promise
  } = withResolvers();
  port.addEventListener('message', resolve, {
    once: true
  });
  const event = await promise;
  // @ts-ignore
  return event.data;
};
const listen$2 = async () => {
  const parentIpcRaw = listen$3();
  signal$2(parentIpcRaw);
  const parentIpc = wrap$5(parentIpcRaw);
  const firstMessage = await waitForFirstMessage(parentIpc);
  if (firstMessage.method !== 'initialize') {
    throw new IpcError('unexpected first message');
  }
  const type = firstMessage.params[0];
  if (type === 'message-port') {
    parentIpc.dispose();
    const port = firstMessage.params[1];
    return port;
  }
  return globalThis;
};
class IpcChildWithModuleWorkerAndMessagePort extends Ipc {
  constructor(port) {
    super(port);
  }
  getData(event) {
    return getData$1(event);
  }
  send(message) {
    this._rawIpc.postMessage(message);
  }
  sendAndTransfer(message, transfer) {
    this._rawIpc.postMessage(message, transfer);
  }
  dispose() {
    if (this._rawIpc.close) {
      this._rawIpc.close();
    }
  }
  onClose(callback) {
    // ignore
  }
  onMessage(callback) {
    this._rawIpc.addEventListener('message', callback);
    this._rawIpc.start();
  }
}
const wrap$4 = port => {
  return new IpcChildWithModuleWorkerAndMessagePort(port);
};
const IpcChildWithModuleWorkerAndMessagePort$1 = {
  __proto__: null,
  listen: listen$2,
  wrap: wrap$4
};
const listen$4 = ({
  port
}) => {
  return port;
};
const signal = port => {
  port.postMessage(readyMessage);
};
class IpcChildWithMessagePort extends Ipc {
  constructor(port) {
    super(port);
  }
  getData(event) {
    return getData$1(event);
  }
  send(message) {
    this._rawIpc.postMessage(message);
  }
  sendAndTransfer(message, transfer) {
    this._rawIpc.postMessage(message, transfer);
  }
  dispose() {
    // ignore
  }
  onClose(callback) {
    // ignore
  }
  onMessage(callback) {
    this._rawIpc.addEventListener('message', callback);
    this._rawIpc.start();
  }
}
const wrap$2 = port => {
  return new IpcChildWithMessagePort(port);
};
const IpcChildWithMessagePort$1 = {
  __proto__: null,
  listen: listen$4,
  signal,
  wrap: wrap$2
};

const getModule = method => {
  switch (method) {
    case ModuleWorker:
      return IpcChildWithModuleWorker$1;
    case ModuleWorkerAndMessagePort:
      return IpcChildWithModuleWorkerAndMessagePort$1;
    case MessagePort:
      return IpcChildWithMessagePort$1;
    default:
      throw new Error('unexpected ipc type');
  }
};

const handleMessagePort = port => {
  const module = getModule(MessagePort);
  const ipc = module.wrap(port);
  handleIpc(ipc);
  ipc.send('ready');
};

const lineMap = Object.create(null);
const setLines = (editorId, lines) => {
  lineMap[editorId] = lines;
};

const getLineInfo = (line, tokens, TokenMap) => {
  const tokensLength = tokens.length;
  let end = 0;
  let start = 0;
  const lineInfo = [];
  for (let i = 0; i < tokensLength; i += 2) {
    const tokenType = tokens[i];
    const tokenLength = tokens[i + 1];
    end += tokenLength;
    const text = line.slice(start, end);
    const className = `Token ${TokenMap[tokenType] || 'Unknown'}`;
    const normalizedText = text;
    lineInfo.push(normalizedText, className);
    start = end;
  }
  return lineInfo;
};

const getLineInfos = (lines, tokenizer, languageId) => {
  const lineInfos = [];
  const {
    tokenizeLine,
    initialLineState,
    hasArrayReturn,
    TokenMap
  } = tokenizer;
  let currentLineState = getInitialLineState(initialLineState);
  for (const line of lines) {
    const result = safeTokenizeLine(languageId, tokenizeLine, line, currentLineState, hasArrayReturn);
    const {
      tokens
    } = result;
    const lineInfo = getLineInfo(line, tokens, TokenMap);
    lineInfos.push(lineInfo);
    currentLineState = result;
  }
  return lineInfos;
};

const NewLine = '\n';

const splitLines = lines => {
  if (!lines) {
    return [];
  }
  return lines.split(NewLine);
};

const tokenizeCodeBlock = async (codeBlock, languageId, tokenizerPath) => {
  await loadTokenizer(languageId, tokenizerPath);
  const tokenizer = getTokenizer(languageId);
  const lines = splitLines(codeBlock);
  const lineInfos = getLineInfos(lines, tokenizer, languageId);
  return lineInfos;
};

const commandMap = {
  [TokenizeCodeBlock]: tokenizeCodeBlock,
  [LoadTokenizer]: loadTokenizer,
  [HandleMessagePort]: handleMessagePort,
  [GetTokensViewport]: getTokensViewport,
  [SetLines]: setLines
};

const listen$1 = async ({
  method
}) => {
  const module = await getModule(method);
  const rawIpc = await module.listen();
  // @ts-ignore
  if (module.signal) {
    // @ts-ignore
    module.signal(rawIpc);
  }
  const ipc = module.wrap(rawIpc);
  return ipc;
};

const listen = async () => {
  registerCommands(commandMap);
  const ipc = await listen$1({
    method: Auto()
  });
  handleIpc(ipc);
};

const main = async () => {
  await listen();
};

main();
